;******************************************************************************************
;'Scroller'. nxSoftware.
;******************************************************************************************
;                                 DEMONSTRATION PROGRAM 1.
;
; This demo simply shows how to set up and administer a basic Scroller control, including the some basic event handlers
; to handle scrolling and painting.
;
; This demo is identical to "Demo 1" except for the fact that this one does not use auto scroll. Consequently we have to use
; our event handler to handle all scrolling messages.
;
; Without a handler to handle scrolling, the scrolling will not actually function. This is how Windows handles scrolling at the API level.
;
; This demo paints some simple scrollable content into the control.
;******************************************************************************************

;Add a reference to the Scroller library.
  IncludePath "../"
    XIncludeFile "Scroller.pbi"
    UseModule Scroller

;Create some global brushes.
  Global gBlueBrush = CreateSolidBrush_(#Blue)
  Global gRedBrush = CreateSolidBrush_(#Red)

Declare EventHandler(this.Scroller, *scrollerEvent.Scroller_EVENT)

Define myScroller.Scroller

OpenWindow(0, 0, 0, 500, 500,"nxSoftware - Scroller demo", #PB_Window_SystemMenu | #PB_Window_SizeGadget | #PB_Window_MinimizeGadget | #PB_Window_MaximizeGadget | #PB_Window_ScreenCentered | #PB_Window_Invisible | #WS_CLIPCHILDREN) 

  ;Create an instance of our Scroller control and check the return value.
  ;The width and height of zero will be sorted by the initial resize.
    myScroller = NewScroller(20, 20, 0, 0, 1000, 1000, #Scroller_FLAT|#Scroller_ADDKEYBOARDCONTROL|#Scroller_NOERASEBKGND)
    If myScroller = 0
      MessageRequester("Scroller!", "Could not create a Scroller object!")
      End
    EndIf

  ;Set an event handler for our scroller. This will handle scrolling and painting events.
    myScroller\SetEventHandler(@EventHandler())

  HideWindow(0, 0)


  Repeat
    Event = WaitWindowEvent()
    Select Event
      Case #PB_Event_SizeWindow
        myScroller\Resize(#PB_Ignore, #PB_Ignore, WindowWidth(0)-40, WindowHeight(0)-40)
    EndSelect
  Until Event = #PB_Event_CloseWindow
  
  ;Destroy our Scroller object.
    myScroller\Destroy()
End


;---------------Event handler

Procedure.i EventHandler(scroller.Scroller, *scrollerEvent.Scroller_EVENT)
  Protected result, hWnd, hdc, ps.PAINTSTRUCT, rc.RECT, scrollX, scrollY, top, bottom, i
  Protected maxX, maxY
  Select *scrollerEvent\eventCode
    Case #Scroller_ONPAINT
      hWnd = GadgetID(scroller\GetID())
      hdc = BeginPaint_(hWnd, ps)
      If hdc
        ;In the case that the scrollable area does not completely cover the client area we have some erasing to perform,
          SetRect_(rc, 0, 0, *scrollerEvent\pageWidth, *scrollerEvent\pageHeight)
          brush = GetStockObject_(#WHITE_BRUSH)
          If *scrollerEvent\documentWidth < *scrollerEvent\pageWidth
            rc\left = *scrollerEvent\documentWidth
            FillRect_(hdc, rc, brush)
            rc\left = 0
          EndIf
          If *scrollerEvent\documentHeight < *scrollerEvent\pageHeight
            rc\top = *scrollerEvent\documentHeight
            FillRect_(hdc, rc, brush)
          EndIf
          DeleteObject_(brush)
        ;Now we alternate strips of red and blue.
          ;Retrieve the current scroll position.
            scroller\GetScrollPositions(@scrollX, @scrollY) 
          ;Determine vertical bounds of the modified (virtual) update region to take the scroll position into account.
            top = ps\rcPaint\top + scrollY
            bottom = ps\rcPaint\bottom + scrollY
          ;Expand to leave the top and bottom at multiples of 50 pixels (since we are painting strips of height 50 pixels).
            top = 50*(top/50)
            bottom = 50*(bottom/50)
          ;Paint the strips; alternate blue and red. Take care not to paint beyond the inner area. The custom erasing above will deal with
          ;any parts of the client area not covered by the inner area.
            For i = top To bottom Step 50
              If i >= *scrollerEvent\documentHeight
                Break
              EndIf
              ;Our final strip may not be the whole 50 pixels (depending upon the size of the inner area).
              t1 = i + 50
              If t1 >= *scrollerEvent\documentHeight
                t1 = *scrollerEvent\documentHeight
              EndIf
              ;Take care to paint in the original update region, not the 'virtual' one obtained by applying the scroll position etc.
                SetRect_(rc, 0, i - scrollY, *scrollerEvent\documentWidth, t1 - scrollY)
              If i/50 = (i/50)>>1<<1
                FillRect_(hdc, rc, gBlueBrush)
              Else
                FillRect_(hdc, rc, gRedBrush)
              EndIf
              ;And place some text in the strip so that we can verify that the scrolling is working.
                rc\left = 5-ScrollX  ;To ensure the text sits 5 pixels from the left of the inner area.
                rc\top + 20
                text$ = "Row " + Str(i/50)
                DrawText_(hdc, text$, Len(text$), rc,0)
            Next
        EndPaint_(hWnd, ps)
      EndIf
      result = #True ;Prevents scroller from invoking the default Windows paint handler.

    Case #Scroller_ONSCROLL
      Select *scrollerEvent\scrollCode
        Case #Scroller_LINELEFTUP
          If *scrollerEvent\blnHorizontalScrollBar
            scroller\GetScrollPositions(@scrollX, 0) 
            scroller\SetScrollX(scrollX-10)
          Else
            scroller\GetScrollPositions(0, @scrollY) 
            scroller\SetScrollY(scrollY-10)
          EndIf
        Case #Scroller_LINERIGHTDOWN
          If *scrollerEvent\blnHorizontalScrollBar
            scroller\GetScrollPositions(@scrollX, 0) 
            scroller\SetScrollX(scrollX+10)
          Else
            scroller\GetScrollPositions(0, @scrollY) 
            scroller\SetScrollY(scrollY+10)
          EndIf
        Case #Scroller_PAGELEFTUP
          If *scrollerEvent\blnHorizontalScrollBar
            scroller\GetScrollPositions(@scrollX, 0) 
            scroller\SetScrollX(scrollX-*scrollerEvent\pageWidth)
          Else
            scroller\GetScrollPositions(0, @scrollY) 
            scroller\SetScrollY(scrollY-*scrollerEvent\pageHeight)
          EndIf
        Case #Scroller_PAGERIGHTDOWN
          If *scrollerEvent\blnHorizontalScrollBar
            scroller\GetScrollPositions(@scrollX, 0) 
            scroller\SetScrollX(scrollX+*scrollerEvent\pageWidth)
          Else
            scroller\GetScrollPositions(0, @scrollY) 
            scroller\SetScrollY(scrollY+*scrollerEvent\pageHeight)
          EndIf
        Case #Scroller_THUMBTRACK
          If *scrollerEvent\blnHorizontalScrollBar
            scroller\SetScrollX(*scrollerEvent\scrollTrackPos)
          Else
            scroller\SetScrollY(*scrollerEvent\scrollTrackPos)
          EndIf
        Case #Scroller_TOP
          scroller\SetScrollX(0)
          scroller\SetScrollY(0)
        Case #Scroller_BOTTOM
          ;Determine the max horizontal and vertical scrolling positions.
          ;These are easy to calculate. E.g. the max vertical scroll position = height of scrollable area - vertical page size.
            maxX = *scrollerEvent\documentWidth - *scrollerEvent\pageWidth
            maxY = *scrollerEvent\documentHeight - *scrollerEvent\pageHeight
          scroller\SetScrollX(maxX)
          scroller\SetScrollY(maxY)
        Case #Scroller_MOUSEWHEEL
          scroller\GetScrollPositions(0, @scrollY) 
          scroller\SetScrollY(scrollY - *scrollerEvent\scrollMouseWheelDelta*10)
      EndSelect
  EndSelect
  ProcedureReturn result
EndProcedure



; IDE Options = PureBasic 5.21 LTS (Windows - x86)
; CursorPosition = 164
; FirstLine = 133
; Folding = -
; EnableAsm
; EnableXP
; Executable = ..\..\..\Users\srod\Documents\Demo1.exe