;******************************************************************************************
;'Scroller'. By Stephen Rodriguez.
;******************************************************************************************
;                                 DEMONSTRATION PROGRAM 2.
;
; In this demo we display an image in a Scroller control and allow it to be scrolled. 
;
; In this demo we set 'auto scroll' which means that the library will automatically process all scrolling messages.
; The alternative is that our event handler would have to do it. 
; Note that this setting will not automatically scroll the control's contents. We rely upon our paint handler for that.
;******************************************************************************************

;Add a reference to the Scroller library.
  IncludePath "../"
    XIncludeFile "Scroller.pbi"
    UseModule Scroller


Declare EventHandler(this.Scroller, *scrollerEvent.Scroller_EVENT)
Define myScroller.Scroller

;---------------Load an image.
  If LoadImage(1, "girl.bmp") = 0
    MessageRequester("Scroller!", "Could not load image!")
    End
  EndIf
  Global imageWidth = ImageWidth(1)
  Global imageHeight = ImageHeight(1)
  

Define myScroller.Scroller

OpenWindow(0, 0, 0, 500, 500,"nxSoftware - Scroller demo", #PB_Window_SystemMenu | #PB_Window_SizeGadget | #PB_Window_MinimizeGadget | #PB_Window_MaximizeGadget | #PB_Window_ScreenCentered | #PB_Window_Invisible | #WS_CLIPCHILDREN) 

  ;Create an instance of our Scroller control and check the return value.
  ;The width and height of zero will be sorted by the initial resize.
    myScroller = NewScroller(20, 20, 0, 0, imageWidth, ImageHeight, #Scroller_FLAT|#Scroller_ADDKEYBOARDCONTROL|#Scroller_NOERASEBKGND)
    If myScroller = 0
      MessageRequester("Scroller!", "Could not create a Scroller object!")
      End
    EndIf

  ;Set auto scroll (saves us the bother of handling scrolling events!) 
  ;Note that this will not automatically scroll the control's contents. We rely upon our paint handler for that.
    myScroller\SetAutoScroll(10) ;scrollStep = 10.

  ;Set an event handler for our scroller. This will handle painting events (not scrolling since we have set auto scroll).
    myScroller\SetEventHandler(@EventHandler())

  HideWindow(0, 0)

  Repeat
    Event = WaitWindowEvent()
    Select Event
      Case #PB_Event_SizeWindow
        myScroller\Resize(#PB_Ignore, #PB_Ignore, WindowWidth(0)-40, WindowHeight(0)-40)
    EndSelect
  Until Event = #PB_Event_CloseWindow
  
  ;Destroy our Scroller object.
    myScroller\Destroy()
End


;---------------Event handler

Procedure.i EventHandler(scroller.Scroller, *scrollerEvent.Scroller_EVENT)
  Protected result, hWnd, hdc, ps.PAINTSTRUCT, rc.RECT, scrollX, scrollY, hdcTemp, oldImage
  Select *scrollerEvent\eventCode
    Case #Scroller_ONPAINT
      hWnd = GadgetID(scroller\GetID())
      hdc = BeginPaint_(hWnd, ps)
      If hdc
        ;In the case that the scrollable area does not completely cover the client area we have some erasing to perform,
          SetRect_(rc, 0, 0, *scrollerEvent\pageWidth, *scrollerEvent\pageHeight)
          brush = GetStockObject_(#WHITE_BRUSH)
          If *scrollerEvent\documentWidth < *scrollerEvent\pageWidth
            rc\left = *scrollerEvent\documentWidth
            FillRect_(hdc, rc, brush)
            rc\left = 0
          EndIf
          If *scrollerEvent\documentHeight < *scrollerEvent\pageHeight
            rc\top = *scrollerEvent\documentHeight
            FillRect_(hdc, rc, brush)
          EndIf
          DeleteObject_(brush)
        ;Retrieve the current scroll position.
          scroller\GetScrollPositions(@scrollX, @scrollY) 
        ;We should really just blt that part of the image corresponding to the update region, but for convenience we blt
        ;the whole thing and simply offset by the scroll positions.
          If ps\rcPaint\left < *scrollerEvent\documentWidth And ps\rcPaint\top < *scrollerEvent\documentHeight
            ;Create a compatible dc to hold the image.
              hdcTemp = CreateCompatibleDC_(hdc)
              If hdcTemp 
                oldImage = SelectObject_(hdcTemp, ImageID(1))
                BitBlt_(hdc, -scrollX, -scrollY, ImageWidth(1), ImageHeight(1), hdcTemp, 0, 0, #SRCCOPY) 
                SelectObject_(hdcTemp, oldImage)
                DeleteDC_(hdcTemp)
              EndIf
          EndIf
        EndPaint_(hWnd, ps)
      EndIf
      result = #True ;Prevents scroller from invoking the default Windows paint handler.
  EndSelect
  ProcedureReturn result
EndProcedure



; IDE Options = PureBasic 5.21 LTS (Windows - x86)
; CursorPosition = 81
; FirstLine = 50
; Folding = -
; EnableAsm
; EnableXP
; Executable = ..\..\..\Users\srod\Documents\Demo1.exe